"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AudioMixer = void 0;
const stream_1 = require("stream");
const os_1 = require("os");
const AudioInput_1 = require("../AudioInput/AudioInput");
const GenerateSilentChunk_1 = require("../Utils/GenerateSilentChunk");
const mixAudioChunks_1 = require("../Utils/mixAudioChunks");
const ChangeVolume_1 = require("../Utils/ChangeVolume");
class AudioMixer extends stream_1.Readable {
    mixerOptions = {
        sampleRate: 48000,
        channels: 1,
        bitDepth: 16,
        endianness: (0, os_1.endianness)(),
        volume: 100,
        highWaterMark: null,
        generateSilent: false,
        silentDuration: null,
        preProcessData: (data) => { return data; },
        delayTime: 1,
        autoClose: false
    };
    inputs = [];
    mixerClosed = false;
    delayTimeValue;
    constructor(mixerArgs) {
        super();
        this.mixerOptions = Object.assign(this.mixerOptions, mixerArgs);
        this.delayTimeValue = typeof this.mixerOptions.delayTime === "number" ? this.mixerOptions.delayTime : this.mixerOptions.delayTime();
        setTimeout(this.loopRead.bind(this), this.delayTimeValue);
        this.once("close", this.close);
    }
    _read() {
        if (this.isPaused())
            return;
        const chunks = this.inputs.map((input) => input.readAudioChunk(this.mixerOptions.highWaterMark)).filter((chunk) => chunk.length > 0);
        if (chunks.length === 0) {
            if (this.mixerOptions.generateSilent) {
                const silentChunk = (0, GenerateSilentChunk_1.default)(this.mixerOptions.highWaterMark ?? (this.mixerOptions.sampleRate * this.mixerOptions.channels / 1000) * Math.max(1, this.mixerOptions.silentDuration ?? this.delayTimeValue));
                this.unshift(silentChunk);
            }
            return;
        }
        const changeVolumeArgs = {
            volume: this.mixerOptions.volume,
            bitDepth: this.mixerOptions.bitDepth,
            endianness: this.mixerOptions.endianness
        };
        const mixedChunkSize = this.mixerOptions.highWaterMark ?? Math.min(...chunks.map(chunk => chunk.length));
        let mixedChunk = (0, mixAudioChunks_1.default)(chunks, mixedChunkSize, this.mixerOptions);
        this.unshift(this.mixerOptions.preProcessData((0, ChangeVolume_1.default)(mixedChunk, changeVolumeArgs)));
    }
    get getOptions() {
        return { ...this.mixerOptions };
    }
    setVolume(volume) {
        this.mixerOptions.volume = volume;
        return this;
    }
    setHighWaterMark(highWaterMark) {
        this.mixerOptions.highWaterMark = highWaterMark;
        return this;
    }
    setgenerateSilent(generateSilent) {
        this.mixerOptions.generateSilent = generateSilent;
        return this;
    }
    setSilentDuration(silentDuration) {
        this.mixerOptions.silentDuration = silentDuration;
        return this;
    }
    setPreProcessData(preProcessData) {
        this.mixerOptions.preProcessData = preProcessData;
        return this;
    }
    setDelayTime(delayTime) {
        this.mixerOptions.delayTime = delayTime;
        return this;
    }
    setAutoClose(autoClose) {
        this.mixerOptions.autoClose = autoClose;
        return this;
    }
    createAudioInput(inputArgs) {
        const audioInput = new AudioInput_1.AudioInput(inputArgs, this.mixerOptions, this.removeAudioInput.bind(this));
        this.inputs.push(audioInput);
        this.emit("addInput");
        return audioInput;
    }
    removeAudioInput(audioInput) {
        const findAudioInput = this.inputs.indexOf(audioInput);
        if (findAudioInput !== -1) {
            this.inputs.splice(findAudioInput, 1);
            this.emit("removeInput");
        }
        if (this.inputs.length === 0 && this.mixerOptions.autoClose)
            return this.close();
        return this;
    }
    close() {
        if (this.mixerClosed && this.inputs.length === 0)
            return this;
        this.inputs.forEach(input => input.close());
        this.destroy();
        this.unshift(null);
        this.mixerClosed = true;
        this.emit("end");
        return this;
    }
    loopRead() {
        if (this.mixerClosed)
            return;
        this._read();
        this.delayTimeValue = typeof this.mixerOptions.delayTime === "number" ? this.mixerOptions.delayTime : this.mixerOptions.delayTime();
        setTimeout(this.loopRead.bind(this), this.delayTimeValue);
    }
}
exports.AudioMixer = AudioMixer;
